from django.shortcuts import render, get_object_or_404
from django.http import HttpResponse, Http404
from .models import *
from django.core.exceptions import ObjectDoesNotExist
from django.http import JsonResponse
from django.views.decorators.csrf import csrf_exempt
import os
from django.db.utils import ProgrammingError
from django.shortcuts import redirect
from django.core.exceptions import ValidationError
import json
from django.views import generic
from django.db.models.aggregates import Count
from django.db.models import Count
# Create your views here.


def index(request):
    user_list = User.objects.filter(isCurrent=1, identity='0').all()
    user_type_list = {"Academic STAFF": [], "Phd": [], "Master": [], "Engineer STAFF": []}
    for u in user_list:
        if u.member_type == 0:
            u.member_type = "Academic STAFF"
        else:
            if u.member_type == 1:
                u.member_type = "Engineer STAFF"
            else:
                if u.member_type == 2:
                    u.member_type = "Phd"
                else:
                    u.member_type = "Master"
        member_type = u.member_type
        if member_type in user_type_list.keys():
            user_type_list[member_type].append(u)
        else:
            user_type_list[member_type] = [u]

    publication_list = Publications.objects.order_by('-date')
    publication_year_list = {}
    for pub in publication_list:
        year = str(pub.date)[0:4]
        pub.date = str(pub.date)[5:10]
        pub.link = pub.link.split('/')[-1]
        print(pub.link)
        if year in publication_year_list.keys():
            publication_year_list[year].append(pub)
        else:
            publication_year_list[year] = [pub]

    bug_list = Bugs.objects.order_by('-datetime')
    cve_year_list = {}
    for cve in bug_list:
        year = str(cve.datetime)[0:4]
        cve.datetime = str(cve.datetime)[5:10]
        print(cve.datetime)
        if year in cve_year_list.keys():
            cve_year_list[year].append(cve)
        else:
            cve_year_list[year] = [cve]

    article_list = Articles.objects.order_by('-editDateTime')
    article_year_list = {}
    for art in article_list:
        year = str(art.editDateTime)[0:4]
        art.editDateTime = str(art.editDateTime)[5:10]
        u = User.objects.filter(userID=art.authorID).values('name')
        art.authorID = u[0]['name']
        if year in article_year_list.keys():
            article_year_list[year].append(art)
        else:
            article_year_list[year] = [art]
    context = {
        'user_type_list': user_type_list,
        'publication_year_list': publication_year_list,
        'cve_year_list': cve_year_list,
        'article_year_list': article_year_list,
    }
    return render(request, 'hunter/index.html', context)


@csrf_exempt
def login(request):
    username = request.POST['username']
    password = request.POST['password']
    user_id=""
    try:
        db_info = User.objects.get(name=username)
    except ObjectDoesNotExist as e:
        response = JsonResponse({"message": "用户不存在"})
        return response
    else:
        if db_info.pwd == password:
            user_id = db_info.userID
            response = JsonResponse({"message": "success", "userid": user_id})
        else:
            response = JsonResponse({"message": "密码错误"})
    return response


def user(request, user_id):
    u = User.objects.get(userID=user_id)
    print(u.name_hanzi)
    username = u.name
    my_publication_year_list = {}
    pub_ids = PubToUser.objects.filter(username=username).all().values("pubID")
    for pub_id in pub_ids:
        print(pub_id)
        pub = get_object_or_404(Publications, pk=pub_id['pubID'])
        year = str(pub.date)[0:4]
        pub.date = str(pub.date)[5:10]
        if year in my_publication_year_list.keys():
            my_publication_year_list[year].append(pub)
        else:
            my_publication_year_list[year] = [pub]

    my_article_list = Articles.objects.filter(authorID=user_id).order_by('-editDateTime')
    my_article_year_list = {}
    for art in my_article_list:
        year = str(art.editDateTime)[0:4]
        art.editDateTime = str(art.editDateTime)[5:10]
        u = User.objects.filter(userID=art.authorID).values('name')
        art.authorID = u[0]['name']
        if year in my_article_year_list.keys():
            my_article_year_list[year].append(art)
        else:
            my_article_year_list[year] = [art]

    my_bug_list = Bugs.objects.filter(owner=username).order_by('-datetime')
    my_cve_year_list = {}
    for cve in my_bug_list:
        year = str(cve.datetime)[0:4]
        cve.datetime = str(cve.datetime)[5:10]
        print(cve.datetime)
        if year in my_cve_year_list.keys():
            my_cve_year_list[year].append(cve)
        else:
            my_cve_year_list[year] = [cve]
    context = {
        'user': u,
        'publication_year_list': my_publication_year_list,
        'cve_year_list': my_cve_year_list,
        'article_year_list': my_article_year_list,
    }

    return render(request, 'hunter/user.html', context)


def article(request, article_id):
    art = get_object_or_404(Articles, pk=article_id)
    return render(request, 'hunter/article.html', {'article': art})


def delete_publication(request, pub_id):
    print(pub_id)
    try:
        PubToUser.objects.filter(pubID=pub_id).delete()
        Publications.objects.filter(pubID=pub_id).delete()
    except ObjectDoesNotExist as e:
        response = JsonResponse({"message": 0})
        return response
    response = JsonResponse({"message": 1})
    return response

@csrf_exempt
def add_publication(request):
    title = request.POST['title']
    print(title)
    authors = request.POST['authors']
    myfile = request.FILES['myfile']
    str='，'.decode('utf-8')
    authors = authors.replace(str, ',')
    journalname = request.POST.get('journalname')
    date = request.POST.get('date')
    index = request.POST.get('index')
    if Publications.objects.last() is None:
        pubID = 1
    else:
        pubID = Publications.objects.last().pubID+1
        BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
        destination = os.path.join(BASE_DIR,'hunter', 'static', 'hunter/publications', myfile.name)
        if os.path.exists(destination):
            os.remove(destination)
        with open(destination,'wb+') as dest:
            for chunk in myfile.chunks():
                dest.write(chunk)
            dest.close()
            link = os.path.join('static','publications',myfile.name)
            try:
                Publications.objects.create(pubID=pubID, title=title,link=link, messages='kidding', authors = authors,journalname=journalname,date=date,indexType=index)
            except ProgrammingError as e:
                return render(request, 'user.html', {'message':'Publications表错误：' + e})
            authorArr = authors.split(',')
            for author in authorArr:
                try:
                    PubToUser.objects.create(pubID = pubID,username=author)
                    return redirect(request, 'index.html', {'uploadMessage': '文件上传成功'})
                except ProgrammingError as e:
                    return render(request, 'index.html', {'uploadError': 'PubToUser数据表错误：' + e})
            # message["warning"] = "上传成功"
            # return HttpResponse(json.dumps(message), content_type='application/json')
    return redirect(request, 'index.html')


